import 'package:flutter/material.dart';
import 'package:flutter_agnes/flutter_agnes.dart';
import 'package:flutter_agnes/src/widgets/dialogs/simple_dialog.dart';
import 'package:flutter_agnes/src/widgets/share/outline_button_widget.dart';
import 'package:asuka/asuka.dart' as asuka;

void main() {
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Agnes Flutter',
      theme: ThemeData(primarySwatch: Colors.blue, accentColor: Colors.white),
      builder: asuka.builder,
      home: MyHomePage(title: 'Agnes Flutter'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  MyHomePage({Key key, this.title}) : super(key: key);
  final String title;

  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> with TickerProviderStateMixin {
  TextEditingController textEditingController = TextEditingController();
  TabController _tabController;

  @override
  void initState() {
    super.initState();
    _tabController = TabController(length: 3, vsync: this);
  }

  Future<MessageModel> getMessage() async {
    try {
      bool error = false;
      if (error) throw 'Simulação de erro!';

      await Future.delayed(const Duration(milliseconds: 2000));
      return MessageModel(code: 1, success: true, message: 'Sucesso!');
    } catch (e) {
      return MessageModel(code: -1, success: false, message: e);
    }
  }

  void _onPressed() async {
    bool result = await asuka.showDialog<bool>(
      barrierDismissible: false,
      builder: (context) => SimpleDialogFuture(
        showMessagePositive: true,
        loadingMessage: 'Aguarde...',
        future: getMessage(),
      ),
    );
    print('SimpleDialogFuture: ${result.toString()}');
  }

  void _onPressedButtonWidget() async {
    bool result = await asuka.showDialog<bool>(
        barrierDismissible: false,
        builder: (context) => SimpleDialogWidget(
              title: 'Message',
              messageModel: MessageModel(code: -1, success: false, message: 'Falha!'),
              onPressed: () => Navigator.pop(context),
            ));
    print('SimpleDialogFuture: ${result.toString()}');
  }

  Future<List<String>> getMonths() async {
    return [
      'January – Janeiro',
      'February – Fevereiro',
      'March – Março',
      'April – Abril',
      'May – Maio',
      'June – Junho',
      'July – Julho',
      'August – Agosto',
      'September – Setembro',
      'October – Outubro',
      'November – Novembro',
      'December – Dezembro'
    ];
  }

  Future<Page> listLoad(int currentPage) async {
    List<Payload> list = [
      Payload(page: 1, item: 'Item 1 '),
      Payload(page: 1, item: 'Item 2'),
      Payload(page: 1, item: 'Item 3'),
      Payload(page: 1, item: 'Item 4'),
      Payload(page: 1, item: 'Item 5'),
      Payload(page: 1, item: 'Item 6'),
      Payload(page: 1, item: 'Item 7'),
      Payload(page: 1, item: 'Item 8'),
      Payload(page: 2, item: 'Item 9'),
      Payload(page: 2, item: 'Item 10'),
      Payload(page: 2, item: 'Item 11'),
      Payload(page: 2, item: 'Item 12'),
      Payload(page: 2, item: 'Item 13'),
      Payload(page: 2, item: 'Item 14'),
      Payload(page: 2, item: 'Item 15'),
      Payload(page: 3, item: 'Item 16'),
      Payload(page: 3, item: 'Item 17'),
      Payload(page: 3, item: 'Item 18'),
      Payload(page: 3, item: 'Item 19'),
      Payload(page: 3, item: 'Item 20'),
      Payload(page: 3, item: 'Item 21'),
      Payload(page: 3, item: 'Item 22'),
    ];

    List modelTests = list.where((element) => element.page == currentPage).toList();
    Page page = Page(currentPage: currentPage, totalPage: 3, payloads: modelTests);
    await Future.delayed(const Duration(milliseconds: 2000));
    return page;
  }

  _listPaginatedWidget() => ListPaginatedWidget.builder(
        listType: ListType.LIST_VIEW,
        loadFuture: listLoad,
        itemCount: (Page response) => response.payloads.length,
        totalPages: (Page response) => response.totalPage,
        pageItems: (Page response) => response.payloads,
        itemsBuilder: (value, int index) {
          Payload payload = value;
          return ListTile(
            title: Text(payload.item),
          );
        },
        loadingWidgetBuilder: () =>
            WaitingResult(label: 'Aguarde...', colorCircular: Theme.of(context).primaryColor, color: Theme.of(context).primaryColor),
        noneResult: NoneResult(
            label: 'Nenhum resultado enconrtado!', background: Theme.of(context).primaryColor, color: Theme.of(context).primaryColor),
      );

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.title),
        bottom: TabBar(
          controller: _tabController,
          tabs: const <Widget>[
            Tab(
              icon: Icon(Icons.category),
              text: 'Widgets and Dialogs',
            ),
            Tab(
              icon: Icon(Icons.pages),
              text: 'Paged list',
            ),
            Tab(
              icon: Icon(Icons.list),
              text: 'Future list',
            ),
          ],
        ),
      ),
      body: TabBarView(
        controller: _tabController,
        children: <Widget>[
          SingleChildScrollView(
            child: Container(
              padding: EdgeInsets.only(top: 20, left: 20, right: 20),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.start,
                children: <Widget>[
                  OutlineButtonWidget(
                    title: 'Simple Dialog Future',
                    icon: Icons.check,
                    width: double.maxFinite,
                    onPressed: _onPressed,
                  ),
                  ButtonWidget(
                      label: 'Simple Dialog Widget',
                      borderRadius: 2,
                      width: double.maxFinite,
                      icon: Icons.check,
                      color: Theme.of(context).accentColor,
                      background: Theme.of(context).primaryColor,
                      onPressed: _onPressedButtonWidget),
                  Row(
                    children: [
                      Expanded(
                          child: TextFormFieldWidget(
                        icon: Icons.message,
                        controller: textEditingController,
                        labelText: 'Text clip oval',
                      )),
                      Expanded(
                          flex: 0,
                          child: ClipOvalWidget(
                              icon: Icons.check,
                              color: Colors.white,
                              background: Theme.of(context).primaryColor,
                              onTap: () {
                                setState(() {
                                  textEditingController.text = 'Test clip oval';
                                  FocusScope.of(context).requestFocus(new FocusNode());
                                });
                              }))
                    ],
                  ),
                ],
              ),
            ),
          ),
          Container(
            padding: EdgeInsets.only(top: 5, left: 20, right: 20, bottom: 20),
            child: Card(
              elevation: 20,
              margin: EdgeInsets.only(top: 20),
              child: Container(
                  padding: EdgeInsets.all(10),
                  width: double.maxFinite,
                  height: 300,
                  child: Column(
                    children: [
                      Container(
                        padding: EdgeInsets.only(top: 10, bottom: 10),
                        child: Text(
                          'Example of paged list',
                          style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold),
                        ),
                      ),
                      Expanded(
                        child: _listPaginatedWidget(),
                      )
                    ],
                  )),
            ),
          ),
          Container(
              padding: EdgeInsets.only(top: 5, left: 20, right: 20, bottom: 20),
              child: Card(
                elevation: 20,
                margin: EdgeInsets.only(top: 20),
                child: Container(
                    padding: EdgeInsets.all(10),
                    height: 400,
                    child: Column(
                      children: [
                        Container(
                          padding: EdgeInsets.only(top: 10, bottom: 10),
                          child: Text(
                            'Example future builder widget',
                            style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold),
                          ),
                        ),
                        Expanded(
                          child: FutureBuilderWidget(
                            loadFuture: getMonths(),
                            builderWidget: (item, index) {
                              return ListTile(
                                title: Text(item),
                              );
                            },
                          ),
                        )
                      ],
                    )),
              )),
        ],
      ),
    );
  }
}

class Payload {
  final int page;
  final String item;

  const Payload({this.page, this.item});
}

class Page {
  final int totalPage;
  final int currentPage;
  final List<Payload> payloads;

  const Page({this.totalPage, this.currentPage, this.payloads});
}
