import 'dart:convert';
import 'package:flutter_agnes/src/models_nortesys/responses/response.dart';
import 'package:http/http.dart' as http;
import '../../../flutter_agnes.dart';

class RequestHelpers<T extends IModel>{

  ObjectCreator<T> creator;
  String token;
  String urlBase;

  RequestHelpers(this.creator, this.token);

  Future<List<T>> getList(String endPoint) async {
    Response<T> response = await getRequest(endPoint);
    //return response.data.listaRetorno;
    return response.data.cargasUteis;
  }

  Future<T> getObject(String endPoint) async {
    Response<T> response = await getRequest(endPoint);
    return response.data.cargasUteis;
  }

  Future<Response<T>> getRequest(String endPoint) async {
    try {
      String url = '$urlBase/$endPoint';
      var uri = Uri.parse(url);
      print(token);
      http.Response response = await http.get(
        uri,
        headers: {
          'Content-type': 'application/json; chrset=UTF-8',
          'Authorization': 'Bearer ${token}',
        },
      );
      print('${response.statusCode} - ${response.body}');
      var dadosJson;
      switch(response.statusCode){
        case 200 :
          dadosJson = json.decode(response.body);
          Response<T> responseObject = Response<T>.fromJson(dadosJson, creator);
          return responseObject;
        case 401 :
          throw MessageModel(code: response.statusCode, success: false, message:  'Sessão expirada!');
      //case 400 :
      //throw MessageModel(code: response.statusCode, success: false, message:  'Bad request: Solicitação mal sucedida!\n${response.body}');
        default :
          dadosJson = json.decode(response.body);
          String message = "";
          for(var element in dadosJson['errors'])
            message += '$element\n';
          throw message;
      }
      /*if (response.statusCode == 200) {
        Response<T> responseObject = Response<T>.fromJson(dadosJson, creator);
        return responseObject;

      } else {
        String message = "";
        for(var element in dadosJson['errors']){
          message += '$element\n';
        }
        throw message;
      }*/
    } catch (e) {
      print(e);
      throw e;
    }
  }

  Future<Response<T>> postRequest(Map<String, dynamic> dataRow, endPoint) async {
    try {
      String url = '$urlBase/$endPoint';
      var uri = Uri.parse(url);
      http.Response response = await http.post(
        uri,
        body: json.encode(dataRow),
        headers: {
          "Content-type": "application/json; chrset=UTF-8",
          'Authorization': 'Bearer $token'
        },
      );
      print('${response.statusCode} - ${response.body}');
      var dadosJson = json.decode(response.body);
      if(dadosJson['success'] == true){
        return Response<T>.fromJson(dadosJson, creator);
      }else{
        String message = "";
        for(var element in dadosJson['errors']){
          message += '$element\n';
        }
        throw message;
      }
    } catch (e) {
      print(e);
      throw e;
    }
  }

  Future<Response<T>> putRequest(Map<String, dynamic> dataRow, endPoint) async {
    try {
      String url = '$urlBase/$endPoint';
      var uri = Uri.parse(url);
      http.Response response = await http.put(
        uri,
        body: json.encode(dataRow),
        headers: {
          "Content-type": "application/json; chrset=UTF-8",
          'Authorization': 'Bearer ${token}'
        },
      );

      print('${response.statusCode} - ${response.body}');

      var dadosJson = json.decode(response.body);

      if(dadosJson['success'] == true){
        return Response<T>.fromJson(dadosJson, creator);
      }else{
        String message = "";
        for(var element in dadosJson['errors']){
          message += '$element\n';
        }
        throw message;
      }
    } catch (e) {
      print(e);
      throw e;
    }
  }

}