
import 'package:flutter/material.dart';
import 'package:flutter_agnes/src/models/message_model.dart';

class SimpleDialogWidget extends StatelessWidget {
  final String title;
  final MessageModel messageModel;
  final VoidCallback onPressed;

  const SimpleDialogWidget({
    Key key,
    this.messageModel,
    this.title,
    this.onPressed,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      contentPadding: EdgeInsets.all(0),
      title: title != null ? Text(title) : null,
      content: Container(
        padding: EdgeInsets.only(left: 20, right: 20, top: 20, bottom: 10),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.all(Radius.circular(5.0)),
        ),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Container(
              alignment: Alignment.center,
              child: Icon(messageModel.success ? Icons.check_circle_outline : Icons.error_outline,
                  color: messageModel.success ? Colors.green : Colors.amber, size: 60),
            ),
            Container(
              padding: EdgeInsets.only(top: 10),
              width: double.maxFinite,
              child: Text(messageModel.message, style: TextStyle(fontSize: 14.0)),
            ),
            Container(
              padding: EdgeInsets.only(top: 10),
              width: double.maxFinite,
              child: OutlineButton.icon(
                color: Theme.of(context).primaryColor,
                textColor: Theme.of(context).primaryColor,
                borderSide: BorderSide(
                  color: Theme.of(context).primaryColor,
                  style: BorderStyle.solid,
                ),
                highlightedBorderColor: Theme.of(context).primaryColor,
                icon: Icon(
                  Icons.check,
                  size: 18,
                ),
                label: Text('Ok'),
                onPressed: onPressed,
              ),
            ),
          ],
        ),
      ),
    );
  }
}
